@@ -3,70 +3,50 @@ from __future__ import annotations
 import datetime as dt
 from decimal import Decimal
 from functools import wraps
-from typing import TYPE_CHECKING
-from typing import Any
-from typing import Callable
-from typing import Literal
-from typing import TypeVar
-from typing import overload
+from typing import TYPE_CHECKING, Any, Callable, Literal, TypeVar, overload
 
-from narwhals._namespace import is_native_arrow
-from narwhals._namespace import is_native_pandas_like
-from narwhals._namespace import is_native_polars
-from narwhals._namespace import is_native_spark_like
-from narwhals.dependencies import get_cudf
-from narwhals.dependencies import get_dask
-from narwhals.dependencies import get_dask_expr
-from narwhals.dependencies import get_modin
-from narwhals.dependencies import get_numpy
-from narwhals.dependencies import get_pandas
-from narwhals.dependencies import get_polars
-from narwhals.dependencies import get_pyarrow
-from narwhals.dependencies import is_cudf_dataframe
-from narwhals.dependencies import is_cudf_series
-from narwhals.dependencies import is_cupy_scalar
-from narwhals.dependencies import is_dask_dataframe
-from narwhals.dependencies import is_duckdb_relation
-from narwhals.dependencies import is_ibis_table
-from narwhals.dependencies import is_modin_dataframe
-from narwhals.dependencies import is_modin_series
-from narwhals.dependencies import is_numpy_scalar
-from narwhals.dependencies import is_pandas_dataframe
-from narwhals.dependencies import is_pandas_like_dataframe
-from narwhals.dependencies import is_pandas_series
-from narwhals.dependencies import is_polars_dataframe
-from narwhals.dependencies import is_polars_lazyframe
-from narwhals.dependencies import is_polars_series
-from narwhals.dependencies import is_pyarrow_chunked_array
-from narwhals.dependencies import is_pyarrow_scalar
-from narwhals.dependencies import is_pyarrow_table
-from narwhals.utils import Version
+from narwhals._namespace import (
+    is_native_arrow,
+    is_native_pandas_like,
+    is_native_polars,
+    is_native_spark_like,
+)
+from narwhals._utils import Version
+from narwhals.dependencies import (
+    get_dask,
+    get_dask_expr,
+    get_numpy,
+    get_pandas,
+    is_cupy_scalar,
+    is_dask_dataframe,
+    is_duckdb_relation,
+    is_ibis_table,
+    is_numpy_scalar,
+    is_pandas_like_dataframe,
+    is_polars_lazyframe,
+    is_polars_series,
+    is_pyarrow_scalar,
+    is_pyarrow_table,
+)
 
 if TYPE_CHECKING:
-    from narwhals.dataframe import DataFrame
-    from narwhals.dataframe import LazyFrame
+    from narwhals.dataframe import DataFrame, LazyFrame
     from narwhals.series import Series
-    from narwhals.typing import DataFrameT
-    from narwhals.typing import IntoDataFrameT
-    from narwhals.typing import IntoFrame
-    from narwhals.typing import IntoFrameT
-    from narwhals.typing import IntoLazyFrameT
-    from narwhals.typing import IntoSeries
-    from narwhals.typing import IntoSeriesT
-    from narwhals.typing import LazyFrameT
-    from narwhals.typing import SeriesT
+    from narwhals.typing import (
+        DataFrameT,
+        IntoDataFrameT,
+        IntoFrame,
+        IntoFrameT,
+        IntoLazyFrameT,
+        IntoSeries,
+        IntoSeriesT,
+        LazyFrameT,
+        SeriesT,
+    )
 
 T = TypeVar("T")
 
-NON_TEMPORAL_SCALAR_TYPES = (
-    bool,
-    bytes,
-    str,
-    int,
-    float,
-    complex,
-    Decimal,
-)
+NON_TEMPORAL_SCALAR_TYPES = (bool, bytes, str, int, float, complex, Decimal)
 TEMPORAL_SCALAR_TYPES = (dt.date, dt.timedelta, dt.time)
 
 
@@ -116,9 +96,9 @@ def to_native(
     Returns:
         Object of class that user started with.
     """
+    from narwhals._utils import validate_strict_and_pass_though
     from narwhals.dataframe import BaseFrame
     from narwhals.series import Series
-    from narwhals.utils import validate_strict_and_pass_though
 
     pass_through = validate_strict_and_pass_though(
         strict, pass_through, pass_through_default=False, emit_deprecation_warning=True
@@ -344,7 +324,7 @@ def from_native(  # noqa: D417
         DataFrame, LazyFrame, Series, or original object, depending
             on which combination of parameters was passed.
     """
-    from narwhals.utils import validate_strict_and_pass_though
+    from narwhals._utils import validate_strict_and_pass_though
 
     pass_through = validate_strict_and_pass_though(
         strict, pass_through, pass_through_default=False, emit_deprecation_warning=True
@@ -375,14 +355,15 @@ def _from_native_impl(  # noqa: C901, PLR0911, PLR0912, PLR0915
     allow_series: bool | None = None,
     version: Version,
 ) -> Any:
-    from narwhals.dataframe import DataFrame
-    from narwhals.dataframe import LazyFrame
+    from narwhals._utils import (
+        _supports_dataframe_interchange,
+        is_compliant_dataframe,
+        is_compliant_lazyframe,
+        is_compliant_series,
+        parse_version,
+    )
+    from narwhals.dataframe import DataFrame, LazyFrame
     from narwhals.series import Series
-    from narwhals.utils import _supports_dataframe_interchange
-    from narwhals.utils import is_compliant_dataframe
-    from narwhals.utils import is_compliant_lazyframe
-    from narwhals.utils import is_compliant_series
-    from narwhals.utils import parse_version
 
     # Early returns
     if isinstance(native_object, (DataFrame, LazyFrame)) and not series_only:
@@ -406,9 +387,8 @@ def _from_native_impl(  # noqa: C901, PLR0911, PLR0912, PLR0915
                 msg = "Cannot only use `series_only` with dataframe"
                 raise TypeError(msg)
             return native_object
-        return DataFrame(
-            native_object.__narwhals_dataframe__(),
-            level="full",
+        return version.dataframe(
+            native_object.__narwhals_dataframe__()._with_version(version), level="full"
         )
     elif is_compliant_lazyframe(native_object):
         if series_only:
@@ -421,9 +401,8 @@ def _from_native_impl(  # noqa: C901, PLR0911, PLR0912, PLR0915
                 msg = "Cannot only use `eager_only` or `eager_or_interchange_only` with lazyframe"
                 raise TypeError(msg)
             return native_object
-        return LazyFrame(
-            native_object.__narwhals_lazyframe__(),
-            level="full",
+        return version.lazyframe(
+            native_object.__narwhals_lazyframe__()._with_version(version), level="full"
         )
     elif is_compliant_series(native_object):
         if not allow_series:
@@ -431,9 +410,8 @@ def _from_native_impl(  # noqa: C901, PLR0911, PLR0912, PLR0915
                 msg = "Please set `allow_series=True` or `series_only=True`"
                 raise TypeError(msg)
             return native_object
-        return Series(
-            native_object.__narwhals_series__(),
-            level="full",
+        return version.series(
+            native_object.__narwhals_series__()._with_version(version), level="full"
         )
 
     # Polars
@@ -552,14 +530,14 @@ def _from_native_impl(  # noqa: C901, PLR0911, PLR0912, PLR0915
     # PySpark
     elif is_native_spark_like(native_object):  # pragma: no cover
         ns_spark = version.namespace.from_native_object(native_object)
-        if series_only:
-            msg = (
-                f"Cannot only use `series_only` with {ns_spark.implementation} DataFrame"
-            )
-            raise TypeError(msg)
-        if eager_only or eager_or_interchange_only:
-            msg = f"Cannot only use `eager_only` or `eager_or_interchange_only` with {ns_spark.implementation} DataFrame"
-            raise TypeError(msg)
+        if series_only or eager_only or eager_or_interchange_only:
+            if not pass_through:
+                msg = (
+                    "Cannot only use `series_only`, `eager_only` or `eager_or_interchange_only` "
+                    f"with {ns_spark.implementation} DataFrame"
+                )
+                raise TypeError(msg)
+            return native_object
         return ns_spark.compliant.from_native(native_object).to_narwhals()
 
     # Interchange protocol
@@ -574,10 +552,18 @@ def _from_native_impl(  # noqa: C901, PLR0911, PLR0912, PLR0915
                 )
                 raise TypeError(msg)
             return native_object
-        return DataFrame(
-            InterchangeFrame(native_object, version=version),
-            level="interchange",
-        )
+        if version is not Version.V1:
+            if pass_through:
+                return native_object
+            msg = (
+                "The Dataframe Interchange Protocol is no longer supported in the main `narwhals` namespace.\n\n"
+                "You may want to:\n"
+                " - Use `narwhals.stable.v1`, where it is still supported.\n"
+                "    - See https://narwhals-dev.github.io/narwhals/backcompat\n"
+                " - Use `pass_through=True` to pass the object through without raising."
+            )
+            raise TypeError(msg)
+        return Version.V1.dataframe(InterchangeFrame(native_object), level="interchange")
 
     elif not pass_through:
         msg = f"Expected pandas-like dataframe, Polars dataframe, or Polars lazyframe, got: {type(native_object)}"
@@ -619,25 +605,20 @@ def get_native_namespace(
     return result.pop()
 
 
-def _get_native_namespace_single_obj(  # noqa: PLR0911
+def _get_native_namespace_single_obj(
     obj: DataFrame[Any] | LazyFrame[Any] | Series[Any] | IntoFrame | IntoSeries,
 ) -> Any:
-    from narwhals.utils import has_native_namespace
+    from contextlib import suppress
+
+    from narwhals._utils import has_native_namespace
+
+    with suppress(TypeError, AssertionError):
+        return Version.MAIN.namespace.from_native_object(
+            obj
+        ).implementation.to_native_namespace()
 
     if has_native_namespace(obj):
         return obj.__native_namespace__()
-    if is_pandas_dataframe(obj) or is_pandas_series(obj):
-        return get_pandas()
-    if is_modin_dataframe(obj) or is_modin_series(obj):  # pragma: no cover
-        return get_modin()
-    if is_pyarrow_table(obj) or is_pyarrow_chunked_array(obj):
-        return get_pyarrow()
-    if is_cudf_dataframe(obj) or is_cudf_series(obj):  # pragma: no cover
-        return get_cudf()
-    if is_dask_dataframe(obj):  # pragma: no cover
-        return get_dask()
-    if is_polars_dataframe(obj) or is_polars_lazyframe(obj) or is_polars_series(obj):
-        return get_polars()
     msg = f"Could not get native namespace from object of type: {type(obj)}"
     raise TypeError(msg)
 
@@ -708,7 +689,7 @@ def narwhalify(
         ... def agnostic_group_by_sum(df):
         ...     return df.group_by("a").agg(nw.col("b").sum())
     """
-    from narwhals.utils import validate_strict_and_pass_though
+    from narwhals._utils import validate_strict_and_pass_though
 
     pass_through = validate_strict_and_pass_though(
         strict, pass_through, pass_through_default=True, emit_deprecation_warning=True
@@ -825,9 +806,4 @@ def _is_pandas_na(obj: Any) -> bool:
     return bool((pd := get_pandas()) and pd.api.types.is_scalar(obj) and pd.isna(obj))
 
 
-__all__ = [
-    "get_native_namespace",
-    "narwhalify",
-    "to_native",
-    "to_py_scalar",
-]
+__all__ = ["get_native_namespace", "narwhalify", "to_native", "to_py_scalar"]
